//@ts-check
import express from "express";
import { Router } from "../Router/Router.mjs";
import customResponseMiddleware from "../middlewares/custom-response.middleware.mjs";

/**
 * @export
 * @class Server
 */
export class Server {
  /**
   * @type {express.Application}
   */
  #app;
  /**
   * Creates an instance of Server.
   *
   * @constructor
   * @param {{
      middlewares?: Array<express.RequestHandler>;
    }} param
   */
  constructor({ middlewares = [] } = { middlewares: [] }) {
    this.#app = express();
    this.#app.use(express.json());
    this.#app.use(express.urlencoded({ extended: true }));
    middlewares.length > 0 && this.#app.use(middlewares);

    this.#app.use(customResponseMiddleware);
  }

  /**
   * Sets several routers
   *
   * @public
   * @param { Router[]} routers
   * @returns {this}
   */
  setRouters(routers) {
    routers.forEach((router) => {
      this.#app.use(router.path, router.router);
    });
    return this;
  }

  /**
   * Sets one router
   *
   * @public
   * @param {Router} router
   * @returns {this}
   */
  setRouter(router) {
    this.#app.use(router.path, router.router);
    return this;
  }

  /**
   * Starts the Server
   *
   * @public
   * @param {number} port
   */
  start(port) {
    this.#app.listen(port, () => {
      console.log(`Server is running on port ${port}`);
    });
  }
}
