/**
 * @typedef {import('express').Request} Request
 * @typedef {import('./custom-response.middleware.mjs').Response} Response
 * @typedef {import('express').NextFunction} NextFunction
 * @typedef {import('../validator/validator.mjs').validator} Validator
 */

/**
 * @callback MiddlewareHandler
 * @param {import('express').Request} req
 * @param {import('../middlewares/custom-response.middleware.mjs').Response} res
 * @param {import('express').NextFunction} next
 * @returns {void}
 */

/**
 * @param {Validator} validator
 * @returns {MiddlewareHandler}
 */
export const validateBody = (validator) => (req, res, next) => {
  const validatedBody = validator(req.body);
  req.body = validatedBody;
  next();
};

/**
 * @param {function} validator
 * @returns {MiddlewareHandler}
 */
export const validateParams = (validator) => (req, res, next) => {
  const validatedParams = validator(req.params);
  req.body = validatedParams;
  next();
};

/**
 * @param {function} validator
 * @returns {MiddlewareHandler}
 */
export const validateQuery = (validator) => (req, res, next) => {
  const validatedQuery = validator(req.query);
  req.query = validatedQuery;
  next();
};
